import { useRef, useReducer, useLayoutEffect, useEffect } from 'react';

function shallowEqual(objA, objB) {
  if (Object.is(objA, objB)) {
    return true;
  }

  if (typeof objA !== 'object' || objA === null || typeof objB !== 'object' || objB === null) {
    return false;
  }

  const keysA = Object.keys(objA);

  if (keysA.length !== Object.keys(objB).length) {
    return false;
  }

  for (let i = 0; i < keysA.length; i++) {
    if (!Object.prototype.hasOwnProperty.call(objB, keysA[i]) || !Object.is(objA[keysA[i]], objB[keysA[i]])) {
      return false;
    }
  }

  return true;
}

const useIsomorphicLayoutEffect = typeof window !== 'undefined' ? useLayoutEffect : useEffect;

const reducer = (state, newState) => newState;

function create(createState) {
  const listeners = new Set();

  const setState = partial => {
    const partialState = typeof partial === 'function' ? partial(state) : partial;

    if (partialState !== state) {
      state = Object.assign({}, state, partialState);
      listeners.forEach(listener => listener(state));
    }
  };

  const getState = () => state; // Optional selector param goes first so we can infer its return type and use
  // it for listener


  const subscribe = (selectorOrListener, listenerOrUndef) => {
    let listener = selectorOrListener; // Existance of second param means a selector was passed in

    if (listenerOrUndef) {
      // We know selector is not type StateListener so it must be StateSelector
      const selector = selectorOrListener;
      let stateSlice = selector(state);

      listener = () => {
        const selectedSlice = selector(state);
        if (!shallowEqual(stateSlice, stateSlice = selectedSlice)) listenerOrUndef(stateSlice);
      };
    }

    listeners.add(listener);
    return () => void listeners.delete(listener);
  };

  const destroy = () => {
    listeners.clear();
  };

  const useStore = (selector, dependencies) => {
    const selectorRef = useRef(selector);
    const depsRef = useRef(dependencies);
    let [stateSlice, dispatch] = useReducer(reducer, state, // Optional third argument but required to not be 'undefined'
    selector); // Need to manually get state slice if selector has changed with no deps or
    // deps exist and have changed

    if (selector && (!dependencies && selector !== selectorRef.current || dependencies && !shallowEqual(dependencies, depsRef.current))) {
      stateSlice = selector(state);
    } // Update refs synchronously after view has been updated


    useIsomorphicLayoutEffect(() => {
      selectorRef.current = selector;
      depsRef.current = dependencies;
    }, dependencies || [selector]);
    useIsomorphicLayoutEffect(() => {
      return selector ? subscribe( // Truthy check because it might be possible to set selectorRef to
      // undefined and call this subscriber before it resubscribes
      () => selectorRef.current ? selectorRef.current(state) : state, dispatch) : subscribe(dispatch); // Only resubscribe to the store when changing selector from function to
      // undefined or undefined to function
    }, [!selector]);
    return stateSlice;
  };

  let api = {
    destroy,
    getState,
    setState,
    subscribe
  };
  let state = createState(setState, getState, api);
  return [useStore, api];
}

export default create;
